/*

Chapter -3
Project 11 Geek Clock
	
*/

//Timer 1 used for interrupt of 1 seconds in CTC Mode
//Timer 0 used for refreshing charlieplexing
//Switches on PB6 and other ends on PA5 and PA7

//Hour Leds
/*                       Led 31 
Led 22        Led 23     Led 30
Led 21        Led 24     Led 29
			  Led 25	 Led 28
			  Led 26     Led 27
	

*/


//Minute Leds
/*
Led 11               Led 20
Led 10        Led 15 Led 19 
Led  9        Led 14 Led 18
Led  8        Led 13 Led 17
Led  7        Led 12 Led 16

*/

//Second Leds
/*
MSB                               LSB
Led 1  Led 2  Led 3  Led 4  Led 5 Led 6
*/



//Schematic
/*
			Enable         Cathode              Variable
Led 1        PA5         	 PA6           secondled[0] 
Led 2        PA5             PA7                     1
Led 3        PA6             PA5                     2  
Led 4        PA6             PA7                     3
Led 5        PA7             PA5                     4
Led 6        PA7             PA6					 5


Led 7        PA0             PA1		   minuteled[0]
Led 8        PA0             PA2					 1
Led 9        PA0             PA3                     2
Led 10       PA0             PA4                     3
Led 11       PA1             PA0                     4
Led 12       PA1             PA2                     5
Led 13		 PA1			 PA3                     6
Led 14		 PA1             PA4                     7
Led 15       PA2             PA0                     8
Led 16	     PA2			 PA1					 9 
Led 17       PA2             PA3                    10
Led 18       PA2             PA4                    11
Led 19       PA3             PA0                    12
Led 20		 PA3             PA1					13

Led 21       PB0             PB1		     hourled[0]
Led 22       PB0             PB2					 1
Led 23       PB0             PB3                     2
Led 24       PB1             PB0                     3
Led 25       PB1             PB2                     4
Led 26       PB1             PB3                     5
Led 27		 PB2			 PB0                     6
Led 28		 PB2             PB1                     7
Led 29       PB2             PB3                     8
Led 30	     PB3			 PB0					 9 
Led 31       PB3             PB1                    10








*/

#include<avr/io.h>
#include<avr/interrupt.h>
#include<avr/pgmspace.h>
#define F_CPU 1843200UL
#include<util/delay.h>


typedef unsigned char u08;


//Function prototypes
u08 call_sec(u08 n); 
u08 call_min(u08 n); 
u08 call_hour(u08 n);
void display(void);
void switch1pressed(void);
void switch2pressed(void);


volatile u08 secondled[6]  = {0,0,0,0,0,0};
volatile u08 minuteled[14] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0};
volatile u08 hourled[11] = {0,0,0,0,0,0,0,0,0,0,0};

//Anode and Cathodes for second LEDs
volatile u08 en_sec[6] PROGMEM= {5,5,6,6,7,7};//Anode
volatile u08 b_sec[6] PROGMEM = {6,7,5,7,5,6};//Cathode

//Anode and Cathodes for minutes LEDs
volatile u08 en_min[14] PROGMEM = {0,0,0,0,1,1,1,1,2,2,2,2,3,3};//Anode
volatile u08 b_min[14] PROGMEM = {1,2,3,4,0,2,3,4,0,1,3,4,0,1};//Cathode

//Anode and Cathodes for hours LEDs
volatile u08 en_hour[11] PROGMEM = {0,0,0,1,1,1,2,2,2,3,3};//Anode
volatile u08 b_hour[11] PROGMEM = {1,2,3,0,2,3,0,1,3,0,1};//Cathode

volatile u08 l_hour,l_min,l_sec;
volatile u08 j_hour=0,j_min=0,j_sec = 0;
volatile u08 sec=0,min=0,hour=0;

volatile u08 display_on=1,display_on1=1,second=0;
volatile u08 random,switch_1_pressed=0,switch_1_released=0,switch_2_pressed=0,switch_2_released=0;
		
int main(void)
{
	
	DDRB = 0x0f;
	PORTB = 0x4f;
	
	DDRA = 0xff;
	PORTA = 0xff;
	
	
	//Timer 1 Initialisation
	TCCR1A = 0x00;
	TCCR1B = 0x8A;//Prescaled by 512 
	OCR1C = 199; //Compare match every 1 seconds																																					//OCR1C = 31; //Compare match every 1 seconds
	
	
	//TIMER 0 Initialisation
	TCCR0= 0x01;
	
	sei();//Enable Global Interrupts
	TIMSK = 1<<TOIE0|1<<TOIE1;//Timer 0 and TIMER1 Interrupt enabled

	
	
	while(1)
	{
		//time to display time
		if(display_on == 1)
		display();
		
		//Switch Code
		
		if(switch_1_released == 1)
		switch1pressed();
		
		if(switch_2_released == 1)
		switch2pressed();
	
	}
}


ISR(TIMER1_OVF1_vect)//Called every one/18 second
{
	second++;
	if(second==18)
	{
		sec++;
		display_on = 1;//update seconds
		if(sec%5==0)
		display_on1 = 1;//update random pattern

		if(sec == 60)
		{
			min++;
			sec = 0;
		}
	
		if(min == 60)
		{
			hour++;
			min = 0;
		}
	
		if(hour == 24)
		{
			hour = 0;
			min = 0;
			sec = 0;
	
		}
		second=0;
	}
}


//Refreshing of charlieplexing
ISR(TIMER0_OVF0_vect)
{	
	
	sei();//So that we dont lose time
	TIMSK &= ~(1<<TOIE0);//Disable timer 0 interrupt so that it doesn't become self calling and hence, recursive
	
	DDRB = 0;
	DDRA = 0;
    
    u08 DDRA_sec,DDRA_min,PORTA_min,PORTA_sec;
	
	l_hour = call_hour(j_hour);
	l_min = call_min(j_min);
	l_sec = call_sec(j_sec);
	
	//Refresh hour
	PORTB = l_hour<<pgm_read_byte(&en_hour[j_hour])|0<<pgm_read_byte(&b_hour[j_hour])|1<<6;

	DDRB = 1<<pgm_read_byte(&en_hour[j_hour])|1<<pgm_read_byte(&b_hour[j_hour]);
	
	//Refresh min
	PORTA_min = l_min<<pgm_read_byte(&en_min[j_min])|0<<pgm_read_byte(&b_min[j_min]);

	DDRA_min = 1<<pgm_read_byte(&en_min[j_min])|1<<pgm_read_byte(&b_min[j_min]);
	
	//Refresh second
	PORTA_sec = l_sec<<pgm_read_byte(&en_sec[j_sec])|0<<pgm_read_byte(&b_sec[j_sec]);

	DDRA_sec = 1<<pgm_read_byte(&en_sec[j_sec])|1<<pgm_read_byte(&b_sec[j_sec]);
	
	
	PORTA = PORTA_min|PORTA_sec;
	
	DDRA = DDRA_min|DDRA_sec;
	
	if(((PORTA&0x80)==0)&&((DDRA&0x80)==0x80)) //If PA7 declared as output and equal to zero
	{
		
		if(!(PINB&0x40))
		{
			switch_1_pressed = 1;
		}
		if(((PINB&0x40)==0x40)&&(switch_1_pressed==1))
		{
			switch_1_released = 1;
			
		}
		
	}
	
	if(!(PORTA&0x20)&&(DDRA&0x20)==0x20) //If PA5 declared as output and equal to zero
	{
		
		if(!(PINB&0x40))
		{
			switch_2_pressed = 1;
		}
		if(((PINB&0x40)==0x40)&&(switch_2_pressed==1))
		{
			switch_2_released = 1;
			
		}
		
	}
	
	
	
	
    
	j_hour++;
	if(j_hour==11)
	j_hour=0;
	j_min++;
	if(j_min==14)
	j_min = 0;
	j_sec++;
	if(j_sec == 6)
	j_sec = 0;
	
	TCNT0 = 0;
	TIFR |= 1<<TOV0; //Clear any pending flag
	TIMSK |= 1<<TOIE0;//Enable timer 0 interrupt
}

u08 call_hour(u08 n)
{
	u08 j;

	j = hourled[n];
	return j;

}

u08 call_min(u08 n)
{
	u08 j;

	j = minuteled[n];
	return j;

}

u08 call_sec(u08 n)
{
	u08 j;

	j = secondled[n];
	return j;

}

void display(void)
{
	display_on = 0;
	
	u08 hour_ten,hour_one,min_ten,min_one;
	
	hour_one = hour%10;
	hour_ten = hour/10;
	min_one = min%10;
	min_ten = min/10;
	
	
	//display seconds
	for(u08 h = 0;h<=5;h++)
	{
		secondled[5-h] = ((sec&(1<<h)) == (1<<h));
	}
	
    if(display_on1==1)//time to change random pattern
    {
		
		display_on1=0;
    
		//All hour and min leds off
        for(u08 o = 0;o<9;o++)
        {
            minuteled[o+5]=0;//clear minutes tens digit
            hourled[o+2] = 0;//clear hour tens digit
            if(o<5)
            minuteled[o] = 0;//clear minutes ones digit
            if(o<2)
            hourled[o] = 0;//clear hour ones digit
        }
    
        //display hour tens digit
		for(u08 o = 0;o<hour_ten;o++)
        {   
		
			//generate random number from 0 to 1
            random = TCNT0;
            random = random%2;
            while(hourled[random] == 1)
            {
                random++;
                if(random==2)
                random = 0;
            }
            hourled[random] = 1;
        }

		//display hour ones digit

		for (u08 o = 0;o<hour_one;o++)
        {
		
            //generate random number from 2 to 10
            random = TCNT0;
            random = random%9+2;
		
            while(hourled[random] == 1)
            {
                random++;
                if(random == 11)
                random = 2;
            }
            hourled[random] = 1;
	
        }
	
		
		//display min tens digit

        for(u08 o = 0;o<min_ten;o++)
        {
		
			//generate random number from 0 to 4
            random = TCNT0;
            random = random%5;
            while(minuteled[random] == 1)
            {
                random++;
                if(random == 5)
                random = 0;
            }
            minuteled[random] = 1;
        }
	
		//display min ones gigit

        for(u08 o = 0;o<min_one;o++)
        {
		
			//generate random numbers from 5 to 13
		
            random = TCNT0;
            random = random%9+5;
            while(minuteled[random] == 1)
            {
                random++;
                if(random == 14)
                random = 5;
            }
            minuteled[random] = 1;
        }
	
    }
}

//update hours
void switch2pressed(void)
{
	
	switch_2_pressed = 0;
	switch_2_released = 0;
    hour++;
	if(hour== 24)
	hour = 0;
    
    display_on=1;
    display_on1=1;
    
}

//update minutes
void switch1pressed(void)
{

	
	switch_1_pressed = 0;
	switch_1_released = 0;
	min++;
	if(min==60)
	min = 0;
    
    display_on=1;
    display_on1=1;
   

}
