/*

Chapter - 4
Project 14 Temperature Plotter

*/

#include<avr/io.h>
#include<avr/interrupt.h>
#include<avr/pgmspace.h>
#define F_CPU 1000000UL
#include<util/delay.h>

#define SCK 6
#define SDIN 2
#define D_C  1
#define t_d 4



//Global Variables

volatile int i=0,k=0,min=1250,max=0,mode=0,dis=1;
volatile unsigned char data[40],flag=0,time=0;

//LCD Prototypes
void initlcd(void);					// Initializes the LCD.
void writecom(char);				// Writes a command.
void writedata(char);				// Writes data to DDRAM to illuminate the pixels.
void clockdata(char);				// Clocks in data into the PCD8544 controller.
void clearram(void);				// Erase the DDRAM contents.
void cursorxy(char,char);			// Position cursor to x,y.
void putcharacter(char);			// Write one character.
void putstr(const char *);			// Writes a string of characters.
void setlcd(void);
void graph1(void);


		 

//DS1820 functions

void output_low(int a)
{
	DDRA |=1<<a;
	PORTA &= ~(1<<a);
}

void output_high(int a)
{
	DDRA|=1<<a;
	PORTA |= (1<<a);
}


void output_float(int a)
{
	DDRA&= ~(1<<a);
	PORTA &= ~(1<<a);
} 

void onewire_reset(void)  // OK if just using a single permanently connected device
{
	output_low(t_d);
	_delay_us( 500 ); // pull 1-wire low for reset pulse
	output_float(t_d); // pull 1-wire floating
	_delay_us( 500 );// wait-out remaining initialisation window. 
	output_float(t_d); // pull 1-wire floating
} 

void onewire_write(unsigned char data)
{
	unsigned char count;

	for (count=0; count<8; count++)
	{
		output_low(t_d);
		_delay_us( 4 ); // pull 1-wire low to initiate write time-slot.
		if((data&0x01)==0x01){output_float(t_d);}
		else {output_low(t_d);}
		data >>=1;
		_delay_us( 60 ); // wait until end of write slot.
		output_float(t_d);
		_delay_us( 2 ); // for more than 1us minimum.
	}
}

unsigned char onewire_read(void)
{
	unsigned char count, data=0;
	char bit;

	for (count=0; count<8; count++)
	{
		output_low(t_d);
		_delay_us( 4 ); // pull 1-wire low to initiate read time-slot.
		output_float(t_d);
		_delay_us( 10 ); // let device state stabilise
		bit = ((PINA&0x10)==0x10);
		data |=bit<<count;
		_delay_us( 60 ); // wait until end of read slot.
	}

	return( data );
} 

int ds1820_read(void)
{
	char busy=0;
	unsigned char temp1,temp2;

	int result;


	onewire_reset();
	onewire_write(0xCC);//Skip Rom Command
	onewire_write(0x44);//Temperature Conversion command

	while (busy == 0)
	busy = onewire_read();
	onewire_reset();
	onewire_write(0xCC);//Skip Rom command
	onewire_write(0xBE);//Read ScratchPad Command
	temp1 = onewire_read();
	temp2 = onewire_read();
	onewire_reset();

	result = temp1*5; // 0.5 deg C resolution //result is ten times actual temperature
	

	return result;
} 

void display(int temperature,char a,char b,char c)
{
	int t;
	t = temperature/1000;
		
		{
			cursorxy(a,b);
			putcharacter(t+48);
			
		}	
		
		
		temperature = temperature%1000;
		
		{	
			cursorxy(a+6,b);
			t = temperature/100;
			putcharacter(t+48);
		}
		
		
		temperature = temperature%100;
		
		{	
			cursorxy(a+12,b);
			t = temperature/10;
			putcharacter(t+48);
		}
		
		temperature = temperature%10;
		
		{	
			cursorxy(a+18,b);
			putcharacter('.');
			t = temperature;
			putcharacter(t+48);
			writedata(0x00);
			writedata(0x03);
			writedata(0x03);
			putcharacter(c);
		}
		
}

const char row1[38] PROGMEM = {
								0xC0,0xE0,0x70,0x30,0x30,
							    0x30,0x30,0x30,0x00,0x00,
								0xC0,0xE0,0x70,0x30,0x30,
								0x30,0x30,0x30,0x00,0x00,
								0xF0,0xF0,0x30,0x30,0x30,
								0x70,0xE0,0xC0,0x00,0x00,
								0xF0,0xF0,0x30,0x30,0x30,
								0x30,0x30,0x30
							   };

const char row2[38] PROGMEM = {
								0x3F,0x7F,0xE0,0xC0,0xC0,
								0xC0,0xC0,0xC0,0x00,0x00,
								0x3F,0x7F,0xE6,0xC6,0xC6,
								0xC6,0xC0,0xC0,0x00,0x00,
								0xFF,0xFF,0xC0,0xC0,0xC0,
								0xE0,0x7F,0x3F,0x00,0x00,
								0x3F,0x7F,0xE0,0xC0,0xC0,
								0xC0,0xC0,0xC0
							   };	

const char graph[5][24]PROGMEM =
								{				
								
								{
								0xFF,0x01,0x01,0x01,
								0x01,0x01,0x01,0x01,
								0x01,0x01,0x01,0xFF,
								0x00,0x00,0x00,0x00,
								0x21,0x21,0x21,0x21,
								0xF9,0x00,0xFF,0xFF
								},

								{
								0xFE,0x02,0x02,0x02,
								0x02,0x1E,0x1E,0x02,
								0x02,0x02,0x02,0xFE,
								0x00,0x00,0x00,0x00,
								0x0A,0x0A,0x4A,0xAA,
								0x1A,0x00,0xFF,0xFF
								},
								{
								0x7E,0xC2,0x82,0x82,
								0x82,0x82,0x82,0x82,
								0x82,0x82,0xC2,0x7E,
								0x00,0x00,0x00,0x00,
								0xFA,0x0A,0x3A,0x0A,
								0xFB,0x00,0xFF,0xFF
								},
								
								{
								0x10,0x10,0x10,0x10,
								0x10,0x10,0x10,0x10,
								0x10,0x10,0x10,0xFE,
								0x00,0x00,0x00,0x00,
								0x08,0x10,0x3F,0x10,
								0x08,0x00,0xFF,0xFF
								},
								

								{
								0x00,0x00,0x00,0x00,
								0x00,0x00,0x00,0x00,
								0x00,0x00,0x00,0x00,
								0x00,0x00,0x00,0x00,
								0x74,0x54,0x57,0x55,
								0x75,0x00,0xFF,0xFF
								}
								};



const char graph0[84]PROGMEM = {
								0x00,0x00,0x00,0x00,
								0x00,0x00,0x00,0x00,
								0x00,0x00,0x00,0x00,
								0x00,0x00,0x00,0x00,
								0xC0,0x40,0x40,0x40,
								0xC0,0x00,0xC0,0xDF,
								0xD1,0xDF,0xC0,0xC0,
								0xC0,0xC0,0xC0,0xC0,
								0xC0,0xC0,0xC0,0xC1,
								0xC1,0xDF,0xC1,0xC1,
								0xC0,0xDF,0xD5,0xD5,
								0xD1,0xC0,0xDF,0xC1,
								0xC2,0xC4,0xC2,0xC1,
								0xDF,0xC0,0xDF,0xC5,
								0xC5,0xC7,0xC0,0xC0,
								0xC4,0xC4,0xC4,0xC4,
								0xD5,0xCE,0xC4,0xC0,
								0xC0,0xC0,0xC0,0xC0,
								0xC0,0xC0,0xDF,0xD5,
								0xD5,0xDD,0xC0,0xDF,
								0xD1,0xDF,0xC0,0xC0
								};


const char table[480] PROGMEM = {	0x00,0x00,0x00,0x00,0x00,	// 20 space		ASCII table for NOKIA LCD: 96 rows * 5 bytes= 480 bytes
							0x00,0x00,0x5f,0x00,0x00,	// 21 !			Note that this is the same set of codes for character you
							0x00,0x07,0x00,0x07,0x00,	// 22 "			would find on a HD44780 based character LCD. :)
							0x14,0x7f,0x14,0x7f,0x14,	// 23 #			Also, given the size of the LCD (84 pixels by 48 pixels),
							0x24,0x2a,0x7f,0x2a,0x12,	// 24 $			the maximum number of characters per row is only 14. :)
							0x23,0x13,0x08,0x64,0x62,	// 25 %
							0x36,0x49,0x55,0x22,0x50,	// 26 &
							0x00,0x05,0x03,0x00,0x00,	// 27 '
							0x00,0x1c,0x22,0x41,0x00,	// 28 (
							0x00,0x41,0x22,0x1c,0x00,	// 29 )
							0x14,0x08,0x3e,0x08,0x14,	// 2a *
							0x08,0x08,0x3e,0x08,0x08,	// 2b +
							0x00,0x50,0x30,0x00,0x00,	// 2c ,
							0x08,0x08,0x08,0x08,0x08,	// 2d -
							0x00,0x60,0x60,0x00,0x00,	// 2e .
							0x20,0x10,0x08,0x04,0x02,	// 2f /
							0x3e,0x51,0x49,0x45,0x3e,	// 30 0
							0x00,0x42,0x7f,0x40,0x00,	// 31 1
							0x42,0x61,0x51,0x49,0x46,	// 32 2
							0x21,0x41,0x45,0x4b,0x31,	// 33 3
							0x18,0x14,0x12,0x7f,0x10,	// 34 4
							0x27,0x45,0x45,0x45,0x39,	// 35 5
							0x3c,0x4a,0x49,0x49,0x30,	// 36 6
							0x01,0x71,0x09,0x05,0x03,	// 37 7
							0x36,0x49,0x49,0x49,0x36,	// 38 8
							0x06,0x49,0x49,0x29,0x1e,	// 39 9
							0x00,0x36,0x36,0x00,0x00,	// 3a :
							0x00,0x56,0x36,0x00,0x00,	// 3b ;
							0x08,0x14,0x22,0x41,0x00,	// 3c <
							0x14,0x14,0x14,0x14,0x14,	// 3d =
							0x00,0x41,0x22,0x14,0x08,	// 3e >
							0x02,0x01,0x51,0x09,0x06,	// 3f ?
							0x32,0x49,0x79,0x41,0x3e,	// 40 @
							0x7e,0x11,0x11,0x11,0x7e,	// 41 A
							0x7f,0x49,0x49,0x49,0x36,	// 42 B
							0x3e,0x41,0x41,0x41,0x22,	// 43 C
							0x7f,0x41,0x41,0x22,0x1c,	// 44 D
							0x7f,0x49,0x49,0x49,0x41,	// 45 E
							0x7f,0x09,0x09,0x09,0x01,	// 46 F
							0x3e,0x41,0x49,0x49,0x7a,	// 47 G
							0x7f,0x08,0x08,0x08,0x7f,	// 48 H
							0x00,0x41,0x7f,0x41,0x00,	// 49 I
							0x20,0x40,0x41,0x3f,0x01,	// 4a J
							0x7f,0x08,0x14,0x22,0x41,	// 4b K
							0x7f,0x40,0x40,0x40,0x40,	// 4c L
							0x7f,0x02,0x0c,0x02,0x7f,	// 4d M
							0x7f,0x04,0x08,0x10,0x7f,	// 4e N
							0x3e,0x41,0x41,0x41,0x3e,	// 4f O
							0x7f,0x09,0x09,0x09,0x06,	// 50 P
							0x3e,0x41,0x51,0x21,0x5e,	// 51 Q
							0x7f,0x09,0x19,0x29,0x46,	// 52 R
							0x46,0x49,0x49,0x49,0x31,	// 53 S
							0x01,0x01,0x7f,0x01,0x01,	// 54 T
							0x3f,0x40,0x40,0x40,0x3f,	// 55 U
							0x1f,0x20,0x40,0x20,0x1f,	// 56 V
							0x3f,0x40,0x38,0x40,0x3f,	// 57 W
							0x63,0x14,0x08,0x14,0x63,	// 58 X
							0x07,0x08,0x70,0x08,0x07,	// 59 Y
							0x61,0x51,0x49,0x45,0x43,	// 5a Z
							0x00,0x7f,0x41,0x41,0x00,	// 5b [
							0x02,0x04,0x08,0x10,0x20,	// 5c Yen Currency Sign
							0x00,0x41,0x41,0x7f,0x00,	// 5d ]
							0x04,0x02,0x01,0x02,0x04,	// 5e ^
							0x40,0x40,0x40,0x40,0x40,	// 5f _
							0x00,0x01,0x02,0x04,0x00,	// 60 `
							0x20,0x54,0x54,0x54,0x78,	// 61 a
							0x7f,0x48,0x44,0x44,0x38,	// 62 b
							0x38,0x44,0x44,0x44,0x20,	// 63 c
							0x38,0x44,0x44,0x48,0x7f,	// 64 d
							0x38,0x54,0x54,0x54,0x18,	// 65 e
							0x08,0x7e,0x09,0x01,0x02,	// 66 f
							0x0c,0x52,0x52,0x52,0x3e,	// 67 g
							0x7f,0x08,0x04,0x04,0x78,	// 68 h
							0x00,0x44,0x7d,0x40,0x00,	// 69 i
							0x20,0x40,0x44,0x3d,0x00,	// 6a j 
							0x7f,0x10,0x28,0x44,0x00,	// 6b k
							0x00,0x41,0x7f,0x40,0x00,	// 6c l
							0x7c,0x04,0x18,0x04,0x78,	// 6d m
							0x7c,0x08,0x04,0x04,0x78,	// 6e n
							0x38,0x44,0x44,0x44,0x38,	// 6f o
							0x7c,0x14,0x14,0x14,0x08,	// 70 p
							0x08,0x14,0x14,0x18,0x7c,	// 71 q
							0x7c,0x08,0x04,0x04,0x08,	// 72 r
							0x48,0x54,0x54,0x54,0x20,	// 73 s
							0x04,0x3f,0x44,0x40,0x20,	// 74 t
							0x3c,0x40,0x40,0x20,0x7c,	// 75 u
							0x1c,0x20,0x40,0x20,0x1c,	// 76 v
							0x3c,0x40,0x30,0x40,0x3c,	// 77 w
							0x44,0x28,0x10,0x28,0x44,	// 78 x
							0x0c,0x50,0x50,0x50,0x3c,	// 79 y
							0x44,0x64,0x54,0x4c,0x44,	// 7a z
							0x00,0x08,0x36,0x41,0x00,	// 7b <
							0x00,0x00,0x7f,0x00,0x00,	// 7c |
							0x00,0x41,0x36,0x08,0x00,	// 7d >
							0x10,0x08,0x08,0x10,0x08,	// 7e Right Arrow 	->
							0x78,0x46,0x41,0x46,0x78};	// 7f Left Arrow	<-


int main(void)
{

	
	
	DDRB = 0x07;
	DDRA = 0b01000000;
	initlcd();
	PORTA = 0b00101011;
	
	
	//CEDT Logo
	cursorxy(23,0);
	for(unsigned char j=0;j<=37;j++)
	{
		writedata(pgm_read_byte(row1+j));
	}
	cursorxy(23,1);
	for(unsigned char j=0;j<=37;j++)
	{
		writedata(pgm_read_byte(row2+j));
	}

	//Timer Initialisation
	
	
	TCCR1A = 0x00;
	TCCR1B = 0x05;
	TIMSK1 = 0x01;
	i=0;
		int temperature,fahrenhite;
	char lk=0;

	
	
	onewire_reset();
	onewire_write(0xCC);//Skip Rom Command
	onewire_write(0x4E);//Write ScratchPad

	onewire_write(125);
	onewire_write(-55); //this should be done for proper working of DS18B20

	onewire_reset();
	onewire_write(0xCC);//Skip Rom Command
	onewire_write(0x48);//Copy ScratchPad
	_delay_ms(500);
	
	sei();
	
	
	while(1)
	{
		
		if(mode==0)
		{
			cli();
			temperature = ds1820_read();
			sei();
			if(temperature>max)
			max = temperature;
			if(temperature<min)
			min = temperature;
			if(lk==10)
			lk=10;
			else lk++;
			fahrenhite = (temperature/5)*(9)+320;
			
			if(flag==1)
			{
				for(unsigned char k=39;k>=1;k--)
				{
					
					data[k] = data[k-1];
				}
				data[0] = temperature/10;
				flag=0;
				if(dis==0)
				graph1();
				
			}
			if(dis==1)
			{
				display(temperature,0,3,'C');
				display(fahrenhite,45,3,'F');
				if(i==5)
				{
					cursorxy(8,4);
					putstr("Max: ");
					display(max,38,4,'C');
			
					cursorxy(8,5);
					putstr("Min: ");
					display(min,38,5,'C');
					
				}
				else if(i==10)
				{
					cursorxy(8,4);
					putstr("Max: ");
					display((max/5)*9+320,38,4,'F');
			
					cursorxy(8,5);
					putstr("Min: ");
					display((min/5)*9+320,38,5,'F');
			
					i=0;
				}
				i++;
			}
		}
		else if(mode==1)
		{
			graph1();
			mode=0;
			
		}
		if(!(PINA&0x01))
		{
			_delay_ms(30);
			while(!(PINA&0x01));
			_delay_ms(30);
			mode = (mode==1)?0:1;
			initlcd();
			if(mode==1&&dis==0)
			mode = 0;
			if(mode==0)
			{
				cursorxy(23,0);
				for(unsigned char j=0;j<=37;j++)
				{
					writedata(pgm_read_byte(row1+j));
				}
				cursorxy(23,1);
				for(unsigned char j=0;j<=37;j++)
				{
					writedata(pgm_read_byte(row2+j));
				}
				dis=1;
			}
			if(mode==1)
			{
				setlcd();
				dis=0;
			}
			
		}
	}
	


}

void initlcd(void)
{
	PORTB = PORTB|1<<0;
	PORTB = PORTB&(~(1<<0));
	_delay_ms(200);
	PORTB = PORTB|1<<0;
	writecom(0x21);		// Activate Chip and H=1.
	writecom(0xD0);		// Set LCD Voltage to about 9V.
	writecom(0x11);		// Adjust voltage bias.
	writecom(0x20);		// Horizontal addressing and H=0.
	writecom(0x09);		// Activate all segments.
	clearram();			// Erase all pixel on the DDRAM.
	writecom(0x08);		// Blank the Display.
	writecom(0x0C);		// Display Normal.
	cursorxy(0,0);		// Cursor Home.
}

void writecom(char command_in)
{
	PORTB = PORTB&(~(1<<D_C));				// Select Command register.
	clockdata(command_in);	// Clock in command bits.
	
}

void writedata(char data_in)
{
	PORTB = PORTB|(1<<D_C);//Select data register
	clockdata(data_in);		// Clock in data bits.
	
}

void clockdata(char bits_in)
{
	int bitcnt;
	for (bitcnt=8; bitcnt>0; bitcnt--)
		{
			PORTA = PORTA&(~(1<<SCK));								// Set Clock Idle level LOW.
			if ((bits_in&0x80)==0x80) {PORTB |=1<<SDIN;}	
			else {PORTB &= ~(1<<SDIN);}
			PORTA |=1<<SCK;								// Data is clocked on the rising edge of SCK.
			bits_in=bits_in<<1;						// Logical shift data by 1 bit left.
		}
}



void cursorxy(char x, char y)
{
	writecom(0x40|(y&0x07));	// Y axis
	writecom(0x80|(x&0x7f));	// X axis
}

void clearram(void)
{
	int ddram;
	cursorxy(0,0);											// Cursor Home.
	for (ddram=504;ddram>0;ddram--)	{writedata(0x00);}		// 6*84 = 504 DDRAM addresses.
}
void putcharacter(char character)
{
	int tablept, count, char_column;

	if ((character<0x20)||(character>0x7f)) {return;}		// Exit function if character is not found.
	tablept = ((5*character) - 160);						// Point to the columns of the character in the table.

	for (count=5;count>0;count--)							// Draw the columns to print the character.
	{
		char_column = pgm_read_byte((table+tablept));
		writedata(char_column);
		tablept++;
	}
	writedata(0x00);										// 1 pixel spacing per character.
}

void putstr(const char *s)
{
	while(*s) {putcharacter(*s++);}					// Points to one ASCII to be written one at a time.
}


void graph1(void)
{
	unsigned char temp=0x80;
	
	for(unsigned char kl=0;kl<=4;kl++)
	{
		for(unsigned char il=0;il<=59;il++)
		{
			cursorxy(il+24,kl+1);
			temp = (data[0+8*kl]==(il+1));
			temp|=((data[1+8*kl]==(il+1))<<1);
			temp|=((data[2+8*kl]==(il+1))<<2);
			temp|=((data[3+8*kl]==(il+1))<<3);
			temp|=((data[4+8*kl]==(il+1))<<4);
			temp|=((data[5+8*kl]==(il+1))<<5);
			temp|=((data[6+8*kl]==(il+1))<<6);
			temp|=((data[7+8*kl]==(il+1))<<7);
			writedata(temp);
			temp=0;
		}
		
	}
	
}

void setlcd(void)
{
	
	for(unsigned char i=0;i<=4;i++)
	{
		for(unsigned char j=0;j<=23;j++)
		{
			cursorxy(j,i+1);
			writedata(pgm_read_byte(&graph[i][j]));
		}
	}
	for(unsigned char j=0;j<=83;j++)
	{
		cursorxy(j,0);
		writedata(pgm_read_byte(&graph0[j]));
		
	}
}

ISR(TIM1_OVF_vect)
{
	time++;
	if(time==9)
	{
		flag=1;
		time=0;
	}

}