/*
CHAPTER - 5 
Project 24 - Contactless Tachometer
*/
#include<avr/io.h>
#include<avr/interrupt.h>
#include<avr/pgmspace.h>
#define F_CPU 8000000UL
#include<util/delay.h>
#include<avr/eeprom.h>

#define SCK 0
#define SDIN 1
#define D_C  2

// MACHINE events
#define MACHINE_TURNING 1
#define MACHINE_STOPPED 0

// Slow rotation ( takes long time to make one round )
// maximum time of one rotation in ms (0.2 Hz = 5000 ms)
#define time_round_max 5000

//Hardware Measurement
// Frequency Measurement on PA2


/*______________NEHUL________________________________________*/

//Global Variables

volatile unsigned int rpm=0;
volatile long double period_total=0;
volatile float frequency=0;
volatile unsigned char no=0,i=1,flag=0,displayon=0;
volatile unsigned int period_actual=0,period_count=0;
//LCD Prototypes
void initlcd(void);					// Initializes the LCD.
void writecom(char);				// Writes a command.
void writedata(char);				// Writes data to DDRAM to illuminate the pixels.
void clockdata(char);				// Clocks in data into the PCD8544 controller.
void clearram(void);				// Erase the DDRAM contents.
void cursorxy(char,char);			// Position cursor to x,y.
void putcharacter(char);			// Write one character.
void putstr(const char *);			// Writes a string of characters.

unsigned char running_condition(void);

void display(unsigned int data,unsigned char what)//Actual times frequency and time
{
	cursorxy(0,what);
	putstr("              ");
	cursorxy(0,what);
	if(what == 4)
	putstr("SPEED");
	else if(what==5)
	putstr("PERIOD");
	int t;
	unsigned char position;
	position = what - 4;
	position = position*6;
	
	t = data/1000;
		
		{
			cursorxy(36+position,what);
			if(t==0)
			putcharacter(' ');
			else
			putcharacter(t+48);
			
		}	
		
		
		data = data%1000;
		
		{	
			cursorxy(42+position,what);
			t = data/100;
			putcharacter(t+48);
		}
		
		
		data = data%100;
		
		{	
			cursorxy(48+position,what);
			t = data/10;
			putcharacter(t+48);
		}
		
		data = data%10;
		
		{	
			cursorxy(54+position,what);
			t=data;
			putcharacter(t+48);
			if(what==4)
			{
				putstr(" RPM");
			}
			else if(what==5)
			{
				putstr(" ms");
			}
			
		}
		
}

const char table[480] PROGMEM = {	0x00,0x00,0x00,0x00,0x00,	// 20 space		ASCII table for NOKIA LCD: 96 rows * 5 bytes= 480 bytes
							0x00,0x00,0x5f,0x00,0x00,	// 21 !			Note that this is the same set of codes for character you
							0x00,0x07,0x00,0x07,0x00,	// 22 "			would find on a HD44780 based character LCD. :)
							0x14,0x7f,0x14,0x7f,0x14,	// 23 #			Also, given the size of the LCD (84 pixels by 48 pixels),
							0x24,0x2a,0x7f,0x2a,0x12,	// 24 $			the maximum number of characters per row is only 14. :)
							0x23,0x13,0x08,0x64,0x62,	// 25 %
							0x36,0x49,0x55,0x22,0x50,	// 26 &
							0x00,0x05,0x03,0x00,0x00,	// 27 '
							0x00,0x1c,0x22,0x41,0x00,	// 28 (
							0x00,0x41,0x22,0x1c,0x00,	// 29 )
							0x14,0x08,0x3e,0x08,0x14,	// 2a *
							0x08,0x08,0x3e,0x08,0x08,	// 2b +
							0x00,0x50,0x30,0x00,0x00,	// 2c ,
							0x08,0x08,0x08,0x08,0x08,	// 2d -
							0x00,0x60,0x60,0x00,0x00,	// 2e .
							0x20,0x10,0x08,0x04,0x02,	// 2f /
							0x3e,0x51,0x49,0x45,0x3e,	// 30 0
							0x00,0x42,0x7f,0x40,0x00,	// 31 1
							0x42,0x61,0x51,0x49,0x46,	// 32 2
							0x21,0x41,0x45,0x4b,0x31,	// 33 3
							0x18,0x14,0x12,0x7f,0x10,	// 34 4
							0x27,0x45,0x45,0x45,0x39,	// 35 5
							0x3c,0x4a,0x49,0x49,0x30,	// 36 6
							0x01,0x71,0x09,0x05,0x03,	// 37 7
							0x36,0x49,0x49,0x49,0x36,	// 38 8
							0x06,0x49,0x49,0x29,0x1e,	// 39 9
							0x00,0x36,0x36,0x00,0x00,	// 3a :
							0x00,0x56,0x36,0x00,0x00,	// 3b ;
							0x08,0x14,0x22,0x41,0x00,	// 3c <
							0x14,0x14,0x14,0x14,0x14,	// 3d =
							0x00,0x41,0x22,0x14,0x08,	// 3e >
							0x02,0x01,0x51,0x09,0x06,	// 3f ?
							0x32,0x49,0x79,0x41,0x3e,	// 40 @
							0x7e,0x11,0x11,0x11,0x7e,	// 41 A
							0x7f,0x49,0x49,0x49,0x36,	// 42 B
							0x3e,0x41,0x41,0x41,0x22,	// 43 C
							0x7f,0x41,0x41,0x22,0x1c,	// 44 D
							0x7f,0x49,0x49,0x49,0x41,	// 45 E
							0x7f,0x09,0x09,0x09,0x01,	// 46 F
							0x3e,0x41,0x49,0x49,0x7a,	// 47 G
							0x7f,0x08,0x08,0x08,0x7f,	// 48 H
							0x00,0x41,0x7f,0x41,0x00,	// 49 I
							0x20,0x40,0x41,0x3f,0x01,	// 4a J
							0x7f,0x08,0x14,0x22,0x41,	// 4b K
							0x7f,0x40,0x40,0x40,0x40,	// 4c L
							0x7f,0x02,0x0c,0x02,0x7f,	// 4d M
							0x7f,0x04,0x08,0x10,0x7f,	// 4e N
							0x3e,0x41,0x41,0x41,0x3e,	// 4f O
							0x7f,0x09,0x09,0x09,0x06,	// 50 P
							0x3e,0x41,0x51,0x21,0x5e,	// 51 Q
							0x7f,0x09,0x19,0x29,0x46,	// 52 R
							0x46,0x49,0x49,0x49,0x31,	// 53 S
							0x01,0x01,0x7f,0x01,0x01,	// 54 T
							0x3f,0x40,0x40,0x40,0x3f,	// 55 U
							0x1f,0x20,0x40,0x20,0x1f,	// 56 V
							0x3f,0x40,0x38,0x40,0x3f,	// 57 W
							0x63,0x14,0x08,0x14,0x63,	// 58 X
							0x07,0x08,0x70,0x08,0x07,	// 59 Y
							0x61,0x51,0x49,0x45,0x43,	// 5a Z
							0x00,0x7f,0x41,0x41,0x00,	// 5b [
							0x02,0x04,0x08,0x10,0x20,	// 5c Yen Currency Sign
							0x00,0x41,0x41,0x7f,0x00,	// 5d ]
							0x04,0x02,0x01,0x02,0x04,	// 5e ^
							0x40,0x40,0x40,0x40,0x40,	// 5f _
							0x00,0x01,0x02,0x04,0x00,	// 60 `
							0x20,0x54,0x54,0x54,0x78,	// 61 a
							0x7f,0x48,0x44,0x44,0x38,	// 62 b
							0x38,0x44,0x44,0x44,0x20,	// 63 c
							0x38,0x44,0x44,0x48,0x7f,	// 64 d
							0x38,0x54,0x54,0x54,0x18,	// 65 e
							0x08,0x7e,0x09,0x01,0x02,	// 66 f
							0x0c,0x52,0x52,0x52,0x3e,	// 67 g
							0x7f,0x08,0x04,0x04,0x78,	// 68 h
							0x00,0x44,0x7d,0x40,0x00,	// 69 i
							0x20,0x40,0x44,0x3d,0x00,	// 6a j 
							0x7f,0x10,0x28,0x44,0x00,	// 6b k
							0x00,0x41,0x7f,0x40,0x00,	// 6c l
							0x7c,0x04,0x18,0x04,0x78,	// 6d m
							0x7c,0x08,0x04,0x04,0x78,	// 6e n
							0x38,0x44,0x44,0x44,0x38,	// 6f o
							0x7c,0x14,0x14,0x14,0x08,	// 70 p
							0x08,0x14,0x14,0x18,0x7c,	// 71 q
							0x7c,0x08,0x04,0x04,0x08,	// 72 r
							0x48,0x54,0x54,0x54,0x20,	// 73 s
							0x04,0x3f,0x44,0x40,0x20,	// 74 t
							0x3c,0x40,0x40,0x20,0x7c,	// 75 u
							0x1c,0x20,0x40,0x20,0x1c,	// 76 v
							0x3c,0x40,0x30,0x40,0x3c,	// 77 w
							0x44,0x28,0x10,0x28,0x44,	// 78 x
							0x0c,0x50,0x50,0x50,0x3c,	// 79 y
							0x44,0x64,0x54,0x4c,0x44,	// 7a z
							0x00,0x08,0x36,0x41,0x00,	// 7b <
							0x00,0x00,0x7f,0x00,0x00,	// 7c |
							0x00,0x41,0x36,0x08,0x00,	// 7d >
							0x10,0x08,0x08,0x10,0x08,	// 7e Right Arrow 	->
							0x78,0x46,0x41,0x46,0x78};	// 7f Left Arrow	<-


int main(void)
{

	
	
	
	DDRB = 0b00001111;
	PORTB =0b00010000;
	initlcd();
	
	
	
	OSCCAL = eeprom_read_byte(0);

	//Timer Initialisation
	
	TCCR0B = 1<<CS02|1<<CS00;//by 1024
	TIMSK = (1 << TOIE0);
	
	//
	GIMSK = 1<<PCIE;
	PCMSK = 1<<PCINT4;
	
	cursorxy(0,5);
	putstr("              ");
	cursorxy(0,6);
	putstr("              ");
	
	
	
	cursorxy(12,1);
	putstr("TACHOMETER");
	
	
	sei();
	
	while(1)
	{
		
		if(running_condition()==MACHINE_TURNING)//do nothing
		{
			
			if(displayon==1)
			{
				cli();
				display(rpm,4);
				display(period_actual,5);
				no=0;period_total=0;
				displayon=0;
				GIFR = 1<<PCIF;//Clear Pending Interrupt
				sei();
			}
			//Everything else handled in interrupts
		}
		else if((flag==0))
		{
			cursorxy(0,2);
			putstr("              ");
			cursorxy(0,3);
			putstr("              ");
			cursorxy(0,4);
			putstr("SPEED    -----");
			cursorxy(0,5);
			putstr("PERIOD   -----");
			flag=1;
			period_total=0;
			no=0;
		}
		
		
	}
	


}

void initlcd(void)
{
	PORTB = PORTB|1<<3;
	PORTB = PORTB&(~(1<<3));
	_delay_ms(200);
	PORTB = PORTB|1<<3;
	writecom(0x21);		// Activate Chip and H=1.
	writecom(0xD3);		// Set LCD Voltage to about 9V.
	writecom(0x13);		// Adjust voltage bias.
	writecom(0x20);		// Horizontal addressing and H=0.
	writecom(0x09);		// Activate all segments.
	clearram();			// Erase all pixel on the DDRAM.
	writecom(0x08);		// Blank the Display.
	writecom(0x0C);		// Display Normal.
	cursorxy(0,0);		// Cursor Home.
}

void writecom(char command_in)
{
	PORTB = PORTB&(~(1<<D_C));				// Select Command register.
	clockdata(command_in);	// Clock in command bits.
	
}

void writedata(char data_in)
{
	PORTB = PORTB|(1<<D_C);//Select data register
	clockdata(data_in);		// Clock in data bits.
	
}

void clockdata(char bits_in)
{
	int bitcnt;
	for (bitcnt=8; bitcnt>0; bitcnt--)
		{
			PORTB = PORTB&(~(1<<SCK));								// Set Clock Idle level LOW.
			if ((bits_in&0x80)==0x80) {PORTB |=1<<SDIN;}	
			else {PORTB &= ~(1<<SDIN);}
			PORTB |=1<<SCK;								// Data is clocked on the rising edge of SCK.
			bits_in=bits_in<<1;						// Logical shift data by 1 bit left.
		}
}



void cursorxy(char x, char y)
{
	writecom(0x40|(y&0x07));	// Y axis
	writecom(0x80|(x&0x7f));	// X axis
}

void clearram(void)
{
	int ddram;
	cursorxy(0,0);											// Cursor Home.
	for (ddram=504;ddram>0;ddram--)	{writedata(0x00);}		// 6*84 = 504 DDRAM addresses.
}
void putcharacter(char character)
{
	int tablept, count, char_column;

	if ((character<0x20)||(character>0x7f)) {return;}		// Exit function if character is not found.
	tablept = ((5*character) - 160);						// Point to the columns of the character in the table.

	for (count=5;count>0;count--)							// Draw the columns to print the character.
	{
		char_column = pgm_read_byte((table+tablept));
		writedata(char_column);
		tablept++;
	}
	writedata(0x00);										// 1 pixel spacing per character.
}

void putstr(const char *s)
{
	while(*s) {putcharacter(*s++);}					// Points to one ASCII to be written one at a time.
}


unsigned char running_condition(void)
{
	// is the actual period possible?
	//if((period_actual>time_round_min)&&
	if(period_count<time_round_max)//)
	{
		return(MACHINE_TURNING);
	}
	else
		return(MACHINE_STOPPED);
}

ISR(TIM0_OVF_vect)
{
	//Every 9.984ms
	TCNT0 = 255-78;
	
	period_total+=9.984;
	if(period_count<5500)
	period_count+=9.984;
	else
	period_count=5500;
	
		
}

ISR(PCINT0_vect)
{
	_delay_us(20);//Let The noise settle
	if(i==0)
	{
		//Face 1 data
		cursorxy(35,3);
		writedata(0b00000000);
		writedata(0b00001100);
		writedata(0b00001100);
		writedata(0b00000001);
		writedata(0b00000011);
		writedata(0b00000111);
		writedata(0b00000111);
		writedata(0b00000011);
		writedata(0b00000001);
		writedata(0b00001100);
		writedata(0b00001100);
		writedata(0b00000000);			
		cursorxy(35,2);
		writedata(0b00000000);
		writedata(0b00110000);
		writedata(0b00110000);
		writedata(0b10000000);
		writedata(0b11000000);
		writedata(0b11100000);
		writedata(0b11100000);
		writedata(0b11000000);
		writedata(0b10000000);
		writedata(0b00110000);
		writedata(0b00110000);	
		writedata(0b00000000);	
		
		i=1;
	}
	else if(i==1)
	{
		//Face 2 data
		cursorxy(35,3);
		writedata(0b00000001);
		writedata(0b00000001);
		writedata(0b00000000);
		writedata(0b00000001);
		writedata(0b00000011);
		writedata(0b00110111);
		writedata(0b00110111);
		writedata(0b00000011);
		writedata(0b00000001);
		writedata(0b00000000);
		writedata(0b00000001);
		writedata(0b00000001);
			
		cursorxy(35,2);
		writedata(0b10000000);
		writedata(0b10000000);
		writedata(0b00000000);
		writedata(0b10000000);
		writedata(0b11000000);
		writedata(0b11101100);
		writedata(0b11101100);
		writedata(0b11000000);
		writedata(0b10000000);
		writedata(0b00000000);
		writedata(0b10000000);	
		writedata(0b10000000);
		i=0;
	}
	//Check if low	
	if(!(PINB&(1<<4)))
	{
		_delay_us(20);//Wait for disturbance to settle
		if(!(PINB&(1<<4))) //Check again if low
		{	
			flag=0;//Display mode on in timeout
			no++;
			period_count=0;
			if(no==6) //6 periods are over
			{
/*************THIS BLOCK CALCULATES THE SPEED AND RPM BASED ON ACTUAL TESTING*************************/			
				if((period_total/(no))<60);//do nothing if twice period less than 60ms Filtering of 2000rpm
				else
				{
					frequency = (12.0/period_total)*1000;
					rpm = frequency*60;//rpm
					period_actual = period_total/12;
					displayon=1;//display on on running mode
					
				}
					period_total=0;no=0;
			}
		}
	}
}

