

/*
CHAPTER - 6
Project 29 - Fridge Alarm Redux
*/



// This file has been prepared for 'AVR TINY EVIL GENIUS BOOK'.
/*! \file main.c
 *
 *
 * 	- File              : main.c	
 *
 * 	- Compiler          : WINAVR
 *
 * 	- Supported devices : ATtiny25, ATtiny 45, ATtiny85 
 *  
 * 	- Description       : The full implementation is included in this file. 
 * 
 * 	$Date: Friday, September 24,2010 UTC $ 
 */
 
#include <avr/io.h>
#define F_CPU 8000000UL
#include <util/delay.h>
#include<avr/pgmspace.h>
#include<avr/interrupt.h>
#include<avr/sleep.h>
#include"fridgealarm.h"

 int main(void)
{
	DDRB|= (1<<LED_PIN|1<<PWM2|1<<PWM1);			//Declare Inputs	
	DDRB &=~(1<<LDR);								//Declare outputs
	
	PORTB|=(1<<LED_PIN);							//LED off
	
	pcint_init();									//Initiate Pin change interrupt
	sei();											//Global Interrupt Enable
	
	while(1)
	{
		if(d_on)									//If Door Remains open for more than 20 second then
		{
			d_on=0;
			timer1_pre();							//Initiate timer1 pll
			timer1_init();							//Initiate timer1 pwm
		}
		
		if(d_alarm)									//Loop to play tone
		{
		 	_delay_us(time);						//time delay to play a sample
		
			OCR1A = pgm_read_byte(sin_wav+sample);	//Variation of Duty Cycle with sinewave samples
			sample++;
			if( sample >= 7 )
			sample = 0;
		}
	}
}


ISR(PCINT0_vect)//pc_int routine
{
	pcint_init();     								//Enabling interrupt
	sei();            								//Global Interrupt Enable
	
	if(!(PINB&(1<<LDR)))  						    //If Door is open, then 
	{
		d_alarm=0;									//initial condition
		sample=0;
		count=0;
		freq=0;
		
		timer0_init();								//Timer 0 initializtion
		sei();										//set interrupt enable
		DDRB|=(1<<LED_PIN);						//LED on
		PORTB&=~(1<<LED_PIN);
	}
	
	else 											//If door is closed
	{	
		d_on=0;										//initial condition
		d_alarm=0;
		all_off();									//all i/o off
 
		powerdown();								//go to power down
		sleep_cpu();
	}
	
}

ISR(TIM0_OVF_vect)
{
	count++;
	
	if((!d_on)&&(count==250))      				//Time count for 20 seconds
	{
			count=0;
			d_on=1;
			d_alarm=1;
	}
	
	if((d_alarm)&&(count==10))						//Time count for tone play time
	{
			count=0;
			
			if(freq<7)
			freq++;
			else	
			freq=0;
			
			freq_run=pgm_read_word((sin_freq+freq));
			time= (125000/freq_run);
			PORTB^=(1<<LED_PIN);
	}
}

void timer1_pre(void)								//Function for Timer1 PLLL Initialization
{
    PLLCSR|=(1<<PLLE);
    _delay_ms(100);
    while(!(PLLCSR&(1<<PLOCK)));
    PLLCSR|=(1<<PCKE);
};

void timer1_init(void)								//Function for Timer1 Pwm iniializtion
{   DDRB|= (1<<PWM2|1<<PWM1);
    TCNT1=0x00;
	OCR1C=0xff;
	TCCR1= (1<<COM1A0)|(1<<CS11)|(1<<PWM1A);
};

void timer0_init(void)								//Function for Timer0 initializtion
{	
	TCNT0=0X00;
	TCCR0B|=(1<<CS00|1<<CS02);//presacle by 1024
	TIMSK|=(1<<TOIE0);
}

void pcint_init(void)								//Function for Pinchange Interrupt initialization
{
	GIMSK|=(1<<PCIE);
	PCMSK|=(1<<PCINT4);
}

void powerdown(void)								//Function for Powerdown mode
{
MCUCR=((1<<SE)|(1<<SM1));
}

void all_off(void)									//Function to switch all I/O off
{
	DDRB&=~(1<<PWM2|1<<PWM1);
	PORTB&=~(1<<PWM2|1<<PWM1);
	PORTB |= (1<<LED_PIN);
	TIMSK &=~(1<<TOIE0);
	TCCR0B&=~(1<<CS00|1<<CS02);
	TCCR1 &=~(1<<COM1A0)|(1<<CS11)|(1<<PWM1A);
}
