/*! This file has been prepared for 'AVR TINY EVIL GENIUS BOOK'.
 *	\file fridgealarm.h
 *	\brief Utilities and functions for controlling the alarm system for fridge
 *	\date 24/9/2010 
 *	This file contains statements and functions to control the behaviour
 *	of the fridgealarm.
 */
 
 #ifndef __FRIDGEALARM_H
	#define __FRIDGEALARM_H
	
	//Macros
	
	/*!	\def LED_PIN
	 * 	\brief LED pin 
	 */
	 #define LED_PIN PB2
     
	/*!	\def PWM1
	 * 	\brief PWM pin 1
	 */
	 #define PWM1 PB0
	 
	/*!	\def PWM2
	 * 	\brief PWM pin 2
	 */
	 #define PWM2 PB1
	 
	/*!	\def LDR
	 * 	\brief LDR pin 
	 */
	 #define LDR PB4
	
	//Prototypes
	
	/*!	\fn  void timer1_pre(void)
	 *	\brief Initialize timer1 clock in synchronisation with pll
	 *	\return void
	 */
	 void timer1_pre(void);
	 
	 
	/*!	\fn  void timer1_init(void)
	 *	\brief Initialize timer1 
	 *	Initialises timer1 with no prescaling and enabling PWM on pin OC1A and 
	 *	and cpmplementary OC1A
	 *	\return void
	 */
	 void timer1_init(void);
	 
	/*!	\fn  void timer0_init(void)
	 *	\brief Initialize timer0 
	 *	Initialises timer0 with 1024 prescaling 
	 *	\return void
	 */
	 void timer0_init(void);
	 
	/*!	\fn  void pcint_init(void)
	 *	\brief Initialize pinchange interrupt on PCINT4
	 *	\return void
	 */
	 void pcint_init(void);
	 
	/*!	\fn  void powerdown(void)
	 *	\brief Initialize powerdown mode
	 *	\return void
	 */
	 void powerdown(void);
	 
	/*!	\fn  void all_off(void)
	 *	\brief Switches off all I/O
	 *	All I/O LED and Speaker are switched off
	 *	\return void
	 */
	 void all_off(void);
	 
	
	//Global Variables
	
	/*!	\char d_on, d_alarm, count, freq, sample
	 *	\brief 
	 *	d_on: door on for 20 seconds
	 *	d_alarm: alarm_on
	 *	count: timer count
	 *	freq: frequency to be played in freq table
	 *	sample: sinewave sample value
	 */
	 volatile unsigned char d_on, d_alarm, count, freq, sample;
	 
	/*!	\int time, freq_run
	 *	\brief 
	 *	time: time period for a sinewave sample to be played
	 *  freq_run: frequency running currently
	 */
	 volatile unsigned int time, freq_run;
	 
	 
	//Lookup Tables
	
	/*!	\brief Sine wave lookup table.
	 *	The sine wave values are specified in this array. It has 8 entries each of 8 bits.
	 *  Entries are of 8 bits each as the PWM Timer/Counter supported in ATTiny45 is 8 bit.
	 */	
	 const char sin_wav[8] PROGMEM=  {128, 218, 255, 218, 128, 38, 1, 38
								      };
	
	/*!	\brief Frequency lookup table.
	 *	The Frequency values specified are specified in this array.
	 */	
	 const int sin_freq[7] PROGMEM=  {200, 300, 400, 500, 600, 700, 800
                                      };
	
#endif