/*
CHAPTER - 7
Project 32 - Batteryless Infrared Remote
*/

#include<avr/io.h>
#include<avr/interrupt.h>
#include<avr/sleep.h>
#define F_CPU 1000000UL
#include<util/delay.h>
#include"remote.h"
 
/*This ISR is used enable the New_key_Pressed variable,
The New_Key_Pressed variable  calls the 
determine_pin_xx function so that the appropriate bit 
according to the key pressed can be trasmitted by IR led*/

ISR(PCINT0_vect)														//Interrupt handler for pin change
{   
	MCUCR &= ~((1<<SE) | (1<<SM1));									//Disable sleep (power down) mode
	PCMSK &= ~((1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2));				//Pin change interrupt is disabled on all pins
	New_Key_Pressed = 1;																									
}

int main(void)
{
	DDRB |= ((1<<COLUMN_1) | (1<<COLUMN_2));									// PINA 1:0 are declared as output
	DDRB &= ~((1<<ROW_3) | (1<<ROW_2) | (1<<ROW_1));						// PINA 4:2 are declared as input
	PORTB |= ((1<<ROW_3) | (1<<ROW_2) | (1<<ROW_1));					// PINA 4:2 are pulled-up high
	GIMSK |= (1<<PCIE);												// Pin change interrupt enabled, PCINT 7:0 pins are enabled
	PCMSK |= ((1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2));				// Pin change interrupt enabled on PIN4:2
	MCUCR |= ((1<<SE) | (1<<SM1));									// Sleep(power down) mode
	
/* #ifdef directives are used to set the protocol to be used 
in the code. Frequency of the PWM according
to that protocol is being made in here*/
/*************************************************
* RC5 PROTOCOL
*************************************************/

	#ifdef RC5
    	
		Tx_bit_RC5[0] = 1;Tx_bit_RC5[1] = 1;																				// start bits
		Tx_bit_RC5[2] = 1;																									// toggle bit
		Tx_bit_RC5[3] = 0;Tx_bit_RC5[4] = 0;Tx_bit_RC5[5] = 0;Tx_bit_RC5[6] = 0;Tx_bit_RC5[7] = 0;						// address bits
		Tx_bit_RC5[8] = 1;Tx_bit_RC5[9] = 1;Tx_bit_RC5[10] = 1;Tx_bit_RC5[11] = 1;Tx_bit_RC5[12] = 1;Tx_bit_RC5[13] = 1;	// command bits
	
		DDRB |= (1<<IR_LED);											//IR led pin is declared as output
		TCCR0A |= ((1<<COM0B1) | (1<<WGM01) | (1<<WGM00));				//Fast PWM mode enabled
		TCCR0B |= ((1<<CS00) | (1<<WGM02));								//Clock frequency 1 MHz(prescalar = 1) 
		OCR0A = 28;														//Approx. 36KHz frequency of PWM
		OCR0B = (OCR0A>>1);												//50% duty cycle
		sei();															//Enable global interrupt

		while(1)
		{ 
			if(New_Key_Pressed == 1)										
				determine_pin_RC5();

			if(Tx == 1)
				transmit_RC5();

			MCUCR |= ((1<<SE) | (1<<SM1));							//Sleep(power down) mode
			PCMSK |= ((1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2));		//Pin change interrupt enabled on PIN4:2
			sleep_cpu();											//Sleep mode ON
		}
   	
	#endif

/*************************************************
* SIRCS PROTOCOL
*************************************************/

	#ifdef SIRCS
   		
		Tx_bit_SIRCS[0] = 0;Tx_bit_SIRCS[1] = 0;Tx_bit_SIRCS[2] = 0;Tx_bit_SIRCS[3] = 0;Tx_bit_SIRCS[4] = 1;			//address bits
		Tx_bit_SIRCS[5] = 0;Tx_bit_SIRCS[6] = 0;Tx_bit_SIRCS[7] = 0;Tx_bit_SIRCS[8] = 0;Tx_bit_SIRCS[9] = 0;			//command bits
		Tx_bit_SIRCS[10] = 0;Tx_bit_SIRCS[11] = 0;																		//command bits

		
		DDRB |= (1<<IR_LED);											//IR led pin is declared as output
		TCCR0A |= ((1<<COM0B1) | (1<<WGM01) | (1<<WGM00));				//Fast PWM mode enabled
		TCCR0B |= ((1<<CS00) | (1<<WGM02));								//Clock frequency 1 MHz(prescalar = 1) 
		OCR0A = 25;														//40KHz frequency of PWM
		OCR0B = (OCR0A>>1);												//50% duty cycle	
		sei();															//Enable global interrupt

		while(1)
		{ 
			if(New_Key_Pressed == 1)
				determine_pin_SIRCS();

			if(Tx == 1)
				transmit_SIRCS();
	
			MCUCR |= ((1<<SE) | (1<<SM1));							//Sleep(power down) mode
			PCMSK |= ((1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2));		//Pin change interrupt enabled on PIN4:2
			sleep_cpu();											//Sleep mode ON
		}

	 #endif

/*************************************************
* SAMSUNG PROTOCOL
*************************************************/

	 #ifdef	SAMSUNG
		
		Tx_bit_Samsung[0] = 1;Tx_bit_Samsung[1] = 1;Tx_bit_Samsung[2] = 1;Tx_bit_Samsung[3] = 0;Tx_bit_Samsung[4] = 0;Tx_bit_Samsung[5] = 0;			//Manufacture's bits
		Tx_bit_Samsung[6] = 0;Tx_bit_Samsung[7] = 0;Tx_bit_Samsung[8] = 1;Tx_bit_Samsung[9] = 1;Tx_bit_Samsung[10] = 1;Tx_bit_Samsung[11] = 0;		//Manufacture's bits
		Tx_bit_Samsung[12] = 0;Tx_bit_Samsung[13] = 0;Tx_bit_Samsung[14] = 0;Tx_bit_Samsung[15] = 0;													//Manufacture's bits
		Tx_bit_Samsung[16] = 0;Tx_bit_Samsung[17] = 0;Tx_bit_Samsung[18] = 0;Tx_bit_Samsung[19] = 0;Tx_bit_Samsung[20] = 0;Tx_bit_Samsung[21] = 0;	//Command bit
		Tx_bit_Samsung[22] = 0;Tx_bit_Samsung[23] = 0;Tx_bit_Samsung[24] = 0;Tx_bit_Samsung[25] = 0;Tx_bit_Samsung[26] = 0;Tx_bit_Samsung[27] = 0;	//Command bit
		Tx_bit_Samsung[28] = 0;Tx_bit_Samsung[29] = 0;Tx_bit_Samsung[30] = 0;Tx_bit_Samsung[31] = 0;													//Command bit

		DDRB |= (1<<IR_LED);											//IR led pin is declared as output
		TCCR0A |= ((1<<COM0B1) | (1<<WGM01) | (1<<WGM00));				//Fast PWM mode enabled
		TCCR0B |= ((1<<CS00) | (1<<WGM02));								//Clock frequency 1 MHz(prescalar = 1) 
		OCR0A = 26;														//Approx. 38KHz frequency of PWM
		OCR0B = (OCR0A>>1);												//50%duty cycle	
		sei();															//Enable global interrupt

		while(1)
		{ 
			if(New_Key_Pressed == 1)
				determine_pin_Samsung();

			if(Tx == 1)
				transmit_Samsung();
	
			MCUCR |= ((1<<SE) | (1<<SM1));							//Sleep(power down) mode
			PCMSK |= ((1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2));		//Pin change interrupt enabled on PIN4:2
			sleep_cpu();											//Sleep mode ON
		}
	
	#endif

/*************************************************
* NEC PROTOCOL
*************************************************/
   
	#ifdef	NEC
	
		Tx_bit_NEC[0] = 0;Tx_bit_NEC[1] = 0;Tx_bit_NEC[2] = 0;Tx_bit_NEC[3] = 0;Tx_bit_NEC[4] = 0;Tx_bit_NEC[5] = 0;Tx_bit_NEC[6] = 0;				//Command bits
		Tx_bit_NEC[7] = 0;Tx_bit_NEC[8] = 0;Tx_bit_NEC[9] = 0;Tx_bit_NEC[10] = 0;Tx_bit_NEC[11] = 0;Tx_bit_NEC[12] = 0;Tx_bit_NEC[13] = 0;			//Command bits
		Tx_bit_NEC[14] = 0;Tx_bit_NEC[15] = 0;																										//Command bits
		Tx_bit_NEC[16] = 1;Tx_bit_NEC[17] = 0;Tx_bit_NEC[18] = 1;Tx_bit_NEC[19] = 0;Tx_bit_NEC[20] = 0;Tx_bit_NEC[21] = 1;Tx_bit_NEC[22] = 1;		//Manufacture's bits
		Tx_bit_NEC[23] = 0;Tx_bit_NEC[24] = 0;Tx_bit_NEC[25] = 1;Tx_bit_NEC[26] = 0;Tx_bit_NEC[27] = 1;Tx_bit_NEC[28] = 1;Tx_bit_NEC[29] = 0;		//Manufacture's bits
		Tx_bit_NEC[30] = 0;Tx_bit_NEC[31] = 1;																										//Manufacture's bits

		DDRB |= (1<<IR_LED);											//IR led pin is declared as output
		TCCR0A |= ((1<<COM0B1) | (1<<WGM01) | (1<<WGM00));				//Fast PWM mode enabled
		TCCR0B |= ((1<<CS00) | (1<<WGM02));								//Clock frequency 1 MHz(prescalar = 1) 
		OCR0A = 26;														//Approx. 38KHz frequency of PWM
		OCR0B = (OCR0A>>1);												//50% duty cycle
		sei();															//Enable global interrupt

		while(1)
		{	 
			if(New_Key_Pressed == 1)
				determine_pin_NEC();

			if(Tx == 1)
				transmit_NEC();
	
			MCUCR |= ((1<<SE) | (1<<SM1));							//Sleep(power down) mode
			PCMSK |= ((1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2));		//Pin change interrupt enabled on PIN4:2
			sleep_cpu();											//Sleep mode ON
		}	

 
	#endif

}

/*Transmit Functions are made to transmit the protocol
bits which are sent from LSB to MSB for SIRCS, NEC, SAMSUNG and MSB to LSB for RC5.
Each protocol has its unique bit pattern which vary in their number of bits, position of command and
address bits*/

void transmit_RC5(void)
{
	while(Tx == 1)
	{
		if(Tx_bit_RC5[i] == 0)
		{
			DDRB |= (1<<IR_LED);											//Enable PWM 
			_delay_us(RC5_ON_PERIOD_ZERO);
			
			PORTB |= (1<<IR_LED);
			DDRB &= ~(1<<IR_LED);											//Disable PWM	
			PORTB |= (1<<IR_LED);
			_delay_us(RC5_OFF_PERIOD_ZERO);
		}

		if(Tx_bit_RC5[i] == 1)
		{
			PORTB |= (1<<IR_LED);
			DDRB &= ~(1<<IR_LED);
			PORTB |= (1<<IR_LED);											//Disable PWM
			_delay_us(RC5_OFF_PERIOD_ONE);
			
			DDRB |= (1<<IR_LED);											//Enable PWM	
			_delay_us(RC5_ON_PERIOD_ONE);
		}
		
		i++;

		if(i == 14)
		{
			i=0;
			Tx = 0;
		}
	}

PCMSK |= ((1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2));
}

 
void transmit_SIRCS(void)
{
	while(Tx == 1)
	{
		for (i=0; i!=3; i++)											//SIRCS needs at least 3 identical bursts separated with a pause of approx. 24 mS
		{
			DDRB |= (1<<IR_LED);											//Enable PWM, start bit
			_delay_us(SIRCS_START_BIT_ON_PERIOD);
			
			PORTB |= (1<<IR_LED);
			DDRB &= ~(1<<IR_LED);											//Disable PWM
			PORTB |= (1<<IR_LED);
			_delay_us(SIRCS_START_BIT_OFF_PERIOD);

			for (j=12; j!=0; j--)										//Data bits *****
			{
				if (Tx_bit_SIRCS[j-1] == 0)
				{
					DDRB |= (1<<IR_LED);									//Enable PWM
					_delay_us(SIRCS_ON_PERIOD_ZERO);
					
					PORTB |= (1<<IR_LED);
					DDRB &= ~(1<<IR_LED); 								//Disable PWM
					PORTB |= (1<<IR_LED);
					_delay_us(SIRCS_OFF_PERIOD_ZERO);
				}
		
				else if (Tx_bit_SIRCS[j-1] == 1)
				{
					DDRB |= (1<<IR_LED);									//Enable PWM
					_delay_us(SIRCS_ON_PERIOD_ONE);
					
					PORTB |= (1<<IR_LED);
					DDRB &= ~(1<<IR_LED); 								//Disable PWM
					PORTB |= (1<<IR_LED);
					_delay_us(SIRCS_OFF_PERIOD_ONE);
				}
			}
	
			_delay_ms(24);
		}

	Tx = 0;
	}

PCMSK |= ((1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2));
}

void transmit_Samsung(void)
{

	while(Tx == 1)	
	{
		DDRB |= (1<<IR_LED);												//Enable PWM, start bit
		_delay_us(SAMSUNG_START_BIT_ON_PERIOD);
		
		PORTB |= (1<<IR_LED);
		DDRB &= ~(1<<IR_LED);
		PORTB |= (1<<IR_LED);												//Disable PWM
		_delay_us(SAMSUNG_START_BIT_OFF_PERIOD);



		for (j=1; j!=33; j++)											//Data bits *****
		{
			if (Tx_bit_Samsung[j-1] == 0)
			{
				DDRB |= (1<<IR_LED);										//Enable PWM
				_delay_us(SAMSUNG_ON_PERIOD_ZERO);
				
				PORTB |= (1<<IR_LED);
				DDRB &= ~(1<<IR_LED); 									//Disable PWM
				PORTB |= (1<<IR_LED);
				_delay_us(SAMSUNG_OFF_PERIOD_ZERO);
			}
		
			else if (Tx_bit_Samsung[j-1] == 1)
			{
				DDRB |= (1<<IR_LED);										//Enable PWM
				_delay_us(SAMSUNG_ON_PERIOD_ONE);
				
				PORTB |= (1<<IR_LED);
				DDRB &= ~(1<<IR_LED); 									//Disable PWM
				PORTB |= (1<<IR_LED);
				_delay_us(SAMSUNG_OFF_PERIOD_ONE);
			}

		}
	
		DDRB |= (1<<IR_LED);												//Enable PWM, stop bit
		_delay_us(SAMSUNG_STOP_BIT_ON_PERIOD);
		
		PORTB |= (1<<IR_LED);
		DDRB &= ~(1<<IR_LED); 											//Disable PWM
		PORTB |= (1<<IR_LED);
		_delay_us(SAMSUNG_STOP_BIT_OFF_PERIOD);
		Tx = 0;
	}

PCMSK |= ((1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2));
}

void transmit_NEC(void)
{
	
	while(Tx == 1)
	{
		DDRB |= (1<<IR_LED);												//Enable PWM, start bit
		_delay_us(NEC_START_BIT_ON_PERIOD);
		
		PORTB |= (1<<IR_LED);
		DDRB &= ~(1<<IR_LED);												//Disable PWM
		PORTB |= (1<<IR_LED);
		_delay_us(NEC_START_BIT_OFF_PERIOD);

		for (j=32; j!=0; j--)											//Data bits *****
		{
			if (Tx_bit_NEC[j-1] == 0)
			{
				DDRB |= (1<<IR_LED);										//Enable PWM
				_delay_us(NEC_ON_PERIOD_ZERO);
				
				PORTB |= (1<<IR_LED);
				DDRB &= ~(1<<IR_LED); 									//Disable PWM
				PORTB |= (1<<IR_LED);
				_delay_us(NEC_OFF_PERIOD_ZERO);
			}
		
			else if (Tx_bit_NEC[j-1] == 1)
			{
				DDRB |= (1<<IR_LED);										//Enable PWM
				_delay_us(NEC_ON_PERIOD_ONE);
				
				PORTB |= (1<<IR_LED);
				DDRB &= ~(1<<IR_LED); 									//Disable PWM
				PORTB |= (1<<IR_LED);
				_delay_us(NEC_OFF_PERIOD_ONE);
			}
		}
	
	DDRB |= (1<<IR_LED);													//Enable PWM, stop bit
	_delay_us(NEC_STOP_BIT_ON_PERIOD);
	
	PORTB |= (1<<IR_LED);
	DDRB &= ~(1<<IR_LED); 												//Disable PWM
	PORTB |= (1<<IR_LED);
	_delay_us(NEC_STOP_BIT_OFF_PERIOD);
	Tx = 0;	
	
	}
PCMSK |= ((1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2));
}

/*With the help of multiplexing, 6 switches have been multiplexed 
and the determine_pin will find out which key is pressed, say mute, volume.. */

void determine_pin_RC5(void)
{

	Tx_bit_RC5[2] ^= 0x01;								//toggle "toggle" bit in RC5 protocol
	char found = 0;  

	PORTB = 0b00011100;
	PORTB |= 0b00000001;

	//Stand By
	if ((PINB & 0x1C) == 0b00010100)
	{
		found =1;
		Tx_bit_RC5[8]=0;Tx_bit_RC5[9]=0;Tx_bit_RC5[10]=1;Tx_bit_RC5[11]=1;Tx_bit_RC5[12]=0;Tx_bit_RC5[13]=0;
	}

	
	//Ch ++
	if ((PINB & 0x1C) == 0b00001100)
	{
		found =1;
		Tx_bit_RC5[8]=1;Tx_bit_RC5[9]=0;Tx_bit_RC5[10]=0;Tx_bit_RC5[11]=0;Tx_bit_RC5[12]=0;Tx_bit_RC5[13]=0;
	}

	//Ch --
	if ((PINB & 0x1C) == 0b00011000)
	{
		found =1;
		Tx_bit_RC5[8]=1;Tx_bit_RC5[9]=0;Tx_bit_RC5[10]=0;Tx_bit_RC5[11]=0;Tx_bit_RC5[12]=0;Tx_bit_RC5[13]=1;
	}
	
	PORTB = 0b00011100;
	PORTB |= 0b00100000;
	
	//Vol --
	if ((PINB & 0x1C) == 0b00010100)
	{
		found =1;
		Tx_bit_RC5[8]=0;Tx_bit_RC5[9]=1;Tx_bit_RC5[10]=0;Tx_bit_RC5[11]=0;Tx_bit_RC5[12]=0;Tx_bit_RC5[13]=1;
	}
	
	//Mute
	if ((PINB & 0x1C) == 0b00011000)
	{
		found =1;
		Tx_bit_RC5[8]=0;Tx_bit_RC5[9]=0;Tx_bit_RC5[10]=1;Tx_bit_RC5[11]=1;Tx_bit_RC5[12]=0;Tx_bit_RC5[13]=1;
	}
	
	//Vol ++
	if ((PINB & 0x1C) == 0b00001100)
	{
		found =1;
		Tx_bit_RC5[8]=0;Tx_bit_RC5[9]=1;Tx_bit_RC5[10]=0;Tx_bit_RC5[11]=0;Tx_bit_RC5[12]=0;Tx_bit_RC5[13]=0;
	}
	
	PORTB = 0b00011100;

    if(found == 1)
		Tx = 1;

	New_Key_Pressed = 0;  
}

void determine_pin_SIRCS(void)
{
	char found = 0;  
	PORTB  = 0b00011100;
	PORTB |= 0b00000001;

	
	//Mute
	if ((PINB & 0x1C) == 0b00010100)
	{
		found =1;
		Tx_bit_SIRCS[5]=0;Tx_bit_SIRCS[6]=0;Tx_bit_SIRCS[7]=1;Tx_bit_SIRCS[8]=0;Tx_bit_SIRCS[9]=1;Tx_bit_SIRCS[10]=0;Tx_bit_SIRCS[11]=0;
		//goto end;
	}
	
	//Ch --
	if ((PINB & 0x1C) == 0b00011000)
	{
		found =1;
		Tx_bit_SIRCS[5]=0;Tx_bit_SIRCS[6]=0;Tx_bit_SIRCS[7]=1;Tx_bit_SIRCS[8]=0;Tx_bit_SIRCS[9]=0;Tx_bit_SIRCS[10]=0;Tx_bit_SIRCS[11]=1;
		//goto end;
	}
	
	//Ch ++
	if(((PINB & 0x1C) == 0b00001100))
	{
		found =1;
		Tx_bit_SIRCS[5]=0;Tx_bit_SIRCS[6]=0;Tx_bit_SIRCS[7]=1;Tx_bit_SIRCS[8]=0;Tx_bit_SIRCS[9]=0;Tx_bit_SIRCS[10]=0;Tx_bit_SIRCS[11]=0;
		goto end;
	}

	
	PORTB = 0b00011100;
	PORTB |= 0b00100000;
	
	//Vol ++
	if ((PINB & 0x1C) == 0b00001100)
	{
		found =1;
		Tx_bit_SIRCS[5]=0;Tx_bit_SIRCS[6]=0;Tx_bit_SIRCS[7]=1;Tx_bit_SIRCS[8]=0;Tx_bit_SIRCS[9]=0;Tx_bit_SIRCS[10]=1;Tx_bit_SIRCS[11]=0;
	
	}
	
	//Stand By
	if ((PINB & 0x1C) == 0b00011000)
	{
		found =1;
		Tx_bit_SIRCS[5]=0;Tx_bit_SIRCS[6]=0;Tx_bit_SIRCS[7]=1;Tx_bit_SIRCS[8]=0;Tx_bit_SIRCS[9]=1;Tx_bit_SIRCS[10]=0;Tx_bit_SIRCS[11]=1;
	
	}
	
	//Vol --
	if ((PINB & 0x1C) == 0b00010100)
	{
		found =1;
		Tx_bit_SIRCS[5]=0;Tx_bit_SIRCS[6]=0;Tx_bit_SIRCS[7]=1;Tx_bit_SIRCS[8]=0;Tx_bit_SIRCS[9]=0;Tx_bit_SIRCS[10]=1;Tx_bit_SIRCS[11]=1;
		//goto end;
	}
	
	
	end : 

	PORTB = 0b00011100;
	if(found == 1)
		Tx = 1;

	New_Key_Pressed = 0;
}

void determine_pin_Samsung(void)

{
	char found = 0;  
	PORTB = 0b00011100;
	PORTB |= 0b00000001;

	//Stand By
	if ((PINB & 0x1C) == 0b00010100)
	{
		found =1;
		Tx_bit_Samsung[16]=0;Tx_bit_Samsung[17]=1;Tx_bit_Samsung[18]=0;Tx_bit_Samsung[19]=0;Tx_bit_Samsung[20]=0;Tx_bit_Samsung[21]=0;
		Tx_bit_Samsung[22]=0;Tx_bit_Samsung[23]=0;Tx_bit_Samsung[24]=1;Tx_bit_Samsung[25]=0;Tx_bit_Samsung[26]=1;Tx_bit_Samsung[27]=1;
		Tx_bit_Samsung[28]=1;Tx_bit_Samsung[29]=1;Tx_bit_Samsung[30]=1;Tx_bit_Samsung[31]=1;
	}
	
	//Ch ++
	if ((PINB & 0x1C) == 0b00011000)
	{
		found =1;
		Tx_bit_Samsung[16]=0;Tx_bit_Samsung[17]=1;Tx_bit_Samsung[18]=0;Tx_bit_Samsung[19]=0;Tx_bit_Samsung[20]=1;Tx_bit_Samsung[21]=0;
		Tx_bit_Samsung[22]=0;Tx_bit_Samsung[23]=0;Tx_bit_Samsung[24]=1;Tx_bit_Samsung[25]=0;Tx_bit_Samsung[26]=1;Tx_bit_Samsung[27]=1;
		Tx_bit_Samsung[28]=0;Tx_bit_Samsung[29]=1;Tx_bit_Samsung[30]=1;Tx_bit_Samsung[31]=1;
	}

	//Ch --
	if ((PINB & 0x1C) == 0b00011000)
	{
		found =1;
		Tx_bit_Samsung[16]=0;Tx_bit_Samsung[17]=0;Tx_bit_Samsung[18]=0;Tx_bit_Samsung[19]=0;Tx_bit_Samsung[20]=1;Tx_bit_Samsung[21]=0;
		Tx_bit_Samsung[22]=0;Tx_bit_Samsung[23]=0;Tx_bit_Samsung[24]=1;Tx_bit_Samsung[25]=1;Tx_bit_Samsung[26]=1;Tx_bit_Samsung[27]=1;
		Tx_bit_Samsung[28]=0;Tx_bit_Samsung[29]=1;Tx_bit_Samsung[30]=1;Tx_bit_Samsung[31]=1;
	}

	PORTB = 0b00011100;
	PORTB |= 0b00100000;
	
	//Vol --
	if ((PINB & 0x1C) == 0b00001100)
	{
		found =1;
		Tx_bit_Samsung[16]=1;Tx_bit_Samsung[17]=1;Tx_bit_Samsung[18]=0;Tx_bit_Samsung[19]=1;Tx_bit_Samsung[20]=0;Tx_bit_Samsung[21]=0;
		Tx_bit_Samsung[22]=0;Tx_bit_Samsung[23]=0;Tx_bit_Samsung[24]=0;Tx_bit_Samsung[25]=0;Tx_bit_Samsung[26]=1;Tx_bit_Samsung[27]=0;
		Tx_bit_Samsung[28]=1;Tx_bit_Samsung[29]=1;Tx_bit_Samsung[30]=1;Tx_bit_Samsung[31]=1;
	}
	//Mute
	if ((PINB & 0x1C) == 0b00011000)
	{
		found =1;
		Tx_bit_Samsung[16]=0;Tx_bit_Samsung[17]=1;Tx_bit_Samsung[18]=0;Tx_bit_Samsung[19]=1;Tx_bit_Samsung[20]=1;Tx_bit_Samsung[21]=0;
		Tx_bit_Samsung[22]=0;Tx_bit_Samsung[23]=0;Tx_bit_Samsung[24]=1;Tx_bit_Samsung[25]=0;Tx_bit_Samsung[26]=1;Tx_bit_Samsung[27]=0;
		Tx_bit_Samsung[28]=0;Tx_bit_Samsung[29]=1;Tx_bit_Samsung[30]=1;Tx_bit_Samsung[31]=1;
	}
	
	//Vol ++
	if ((PINB & 0x1C) == 0b00101000)
	{
		found =1;
		Tx_bit_Samsung[16]=1;Tx_bit_Samsung[17]=1;Tx_bit_Samsung[18]=1;Tx_bit_Samsung[19]=0;Tx_bit_Samsung[20]=0;Tx_bit_Samsung[21]=0;
		Tx_bit_Samsung[22]=0;Tx_bit_Samsung[23]=0;Tx_bit_Samsung[24]=0;Tx_bit_Samsung[25]=0;Tx_bit_Samsung[26]=0;Tx_bit_Samsung[27]=1;
		Tx_bit_Samsung[28]=1;Tx_bit_Samsung[29]=1;Tx_bit_Samsung[30]=1;Tx_bit_Samsung[31]=1;
	}
	
	
	PORTB = 0b00011100;
	if(found == 1)
		Tx = 1;

	New_Key_Pressed = 0;

}

void determine_pin_NEC(void)

{
	char found = 0;  
	PORTB = 0b00011100;
	PORTB |= 0b00000001;

	//Stand By
	if ((PINB & 0x1C) == 0b00010100)
	{
		found =1;
		Tx_bit_NEC[8]=0;Tx_bit_NEC[9]=0;Tx_bit_NEC[10]=0;Tx_bit_NEC[11]=0;Tx_bit_NEC[12]=1;Tx_bit_NEC[13]=0;Tx_bit_NEC[14]=0;
		Tx_bit_NEC[15]=0;Tx_bit_NEC[0]=1;Tx_bit_NEC[1]=1;Tx_bit_NEC[2]=1;Tx_bit_NEC[3]=1;Tx_bit_NEC[4]=0;Tx_bit_NEC[5]=1;
		Tx_bit_NEC[6]=1;Tx_bit_NEC[7]=1;
	}

	
	//Ch ++
	if ((PINB & 0x1C) == 0b00001100)
	{
		found =1;
		Tx_bit_NEC[8]=0;Tx_bit_NEC[9]=0;Tx_bit_NEC[10]=0;Tx_bit_NEC[11]=0;Tx_bit_NEC[12]=0;Tx_bit_NEC[13]=0;Tx_bit_NEC[14]=0;
		Tx_bit_NEC[15]=0;Tx_bit_NEC[0]=1;Tx_bit_NEC[1]=1;Tx_bit_NEC[2]=1;Tx_bit_NEC[3]=1;Tx_bit_NEC[4]=1;Tx_bit_NEC[5]=1;
		Tx_bit_NEC[6]=1;Tx_bit_NEC[7]=1;
	}

	//Ch --
	if ((PINB & 0x1C) == 0b00011000)
	{
		found =1;
		Tx_bit_NEC[8]=0;Tx_bit_NEC[9]=0;Tx_bit_NEC[10]=0;Tx_bit_NEC[11]=0;Tx_bit_NEC[12]=0;Tx_bit_NEC[13]=0;Tx_bit_NEC[14]=0;
		Tx_bit_NEC[15]=1;Tx_bit_NEC[0]=1;Tx_bit_NEC[1]=1;Tx_bit_NEC[2]=1;Tx_bit_NEC[3]=1;Tx_bit_NEC[4]=1;Tx_bit_NEC[5]=1;
		Tx_bit_NEC[6]=1;Tx_bit_NEC[7]=0;
	}
	
	
	PORTB = 0b00011100;
	PORTB |= 0b00100000;
	
	//Vol --
	if ((PINB & 0x1C) == 0b00010100)
	{
		found =1;
		Tx_bit_NEC[8]=0;Tx_bit_NEC[9]=0;Tx_bit_NEC[10]=0;Tx_bit_NEC[11]=0;Tx_bit_NEC[12]=0;Tx_bit_NEC[13]=0;Tx_bit_NEC[14]=1;
		Tx_bit_NEC[15]=1;Tx_bit_NEC[0]=1;Tx_bit_NEC[1]=1;Tx_bit_NEC[2]=1;Tx_bit_NEC[3]=1;Tx_bit_NEC[4]=1;Tx_bit_NEC[5]=1;
		Tx_bit_NEC[6]=0;Tx_bit_NEC[7]=0;
	}

	//Mute
	if ((PINB & 0x1C) == 0b00011000)
	{
		found =1;
		Tx_bit_NEC[8]=0;Tx_bit_NEC[9]=0;Tx_bit_NEC[10]=0;Tx_bit_NEC[11]=0;Tx_bit_NEC[12]=1;Tx_bit_NEC[13]=0;Tx_bit_NEC[14]=0;
		Tx_bit_NEC[15]=1;Tx_bit_NEC[0]=1;Tx_bit_NEC[1]=1;Tx_bit_NEC[2]=1;Tx_bit_NEC[3]=1;Tx_bit_NEC[4]=0;Tx_bit_NEC[5]=1;
		Tx_bit_NEC[6]=1;Tx_bit_NEC[7]=0;
	}
	
	//Vol ++
	if ((PINB & 0x1C) == 0b00001100)
	{
		found =1;
		Tx_bit_NEC[8]=0;Tx_bit_NEC[9]=0;Tx_bit_NEC[10]=0;Tx_bit_NEC[11]=0;Tx_bit_NEC[12]=0;Tx_bit_NEC[13]=0;Tx_bit_NEC[14]=1;
		Tx_bit_NEC[15]=0;Tx_bit_NEC[0]=1;Tx_bit_NEC[1]=1;Tx_bit_NEC[2]=1;Tx_bit_NEC[3]=1;Tx_bit_NEC[4]=1;Tx_bit_NEC[5]=1;
		Tx_bit_NEC[6]=0;Tx_bit_NEC[7]=1;
	}

	PORTB = 0b00011100;
	if(found == 1)
		Tx = 1;

	New_Key_Pressed = 0;
}
