/*
CHAPTER - 2 
Project 4 - Random color and Music Generator
*/




#include <avr/io.h>
#include <avr/interrupt.h>
#include<avr/pgmspace.h>

//Delay function is calculated on the basis of 9.6 MHz
//It has been simulated on AVRstudio4
//31 us = 43



//delay values for sound
const int d[] PROGMEM={43,85,171,343,685,914,1371,1828,2743};

//PWM values for color 
// 0 - On 9 - OFF
const unsigned char k[16] PROGMEM ={7,6,2,5,4,7,9,1,5,2,6,4,0,9,0,9};//blue
const unsigned char l[16] PROGMEM ={5,5,3,3,7,2,3,4,6,3,0,5,0,0,9,9};//red
const unsigned char m[16] PROGMEM ={9,6,4,2,1,0,4,9,1,4,2,1,0,9,9,0};//green

//Some variables required. 
volatile unsigned char e=0,blue,red,green;
volatile unsigned char i=0,a,j=0;
volatile unsigned int t=0;
volatile unsigned int reg,bit;

//Functions prototype

//This function is called to switch on the LED at the 
//Required stage of PWM cycle
void abc(void);

//Delay function
void delay(int);


int main(void) 
{
	
	//Timer Initialisation
	
	TCCR0A = 0x00; //No PWM
	TCCR0B = 0x03; //Prescaling
	TIMSK0 = 1<<TOIE0; //Interrupt enable
	
	//ADC is just used to get the initial seed for Linear Feedback Shift Register
	//ADC Initialisation
	ADMUX = 0b00000010;
	ADCSRA = 0b11101111;
	ADCSRB = 0x00;
	
	//Global Interrupts enabled
	sei();
	
	DDRB=0x0f;//Set PORTB0-3 pin as output
	PORTB = 0x0f;
	
	
    
	while (1) 
	{
	
	//Wait for the seed from ADC 
	
		if(i==4)
		{
			
			//This is the software code for LFSR
			bit = (reg & 0x0001) ^((reg & 0x0004) >> 2) ^((reg & 0x0008) >> 3)^((reg & 0x0020) >> 5);
			reg = (reg >> 1) | (bit << 15);
			
			//Sound generating code
			PORTB |= 1<<3;
			delay(t);
			PORTB &= ~(1<<3);
			delay(t);
	   
		}
   
	}
}

//ADC interrupt service routine. Gives its fourth reading as seed
ISR(ADC_vect)
{
	a = ADCL;
	i++;
	if(i==4)
	{
		reg = a;
		ADCSRA &= ~(1<<ADIE);//ADC job is over
	}
}

//Timer0 overflow ISR
ISR(TIM0_OVF_vect)
{	

	//for color
	if(e==9)
	{
		e=0;
		//Start of new cycle
		PORTB = PORTB|(1<<2)|(1<<1)|(1<<0);
	}
	abc();
	
	//Time to change
	//Approximately half a second
	j++;
	if(j==128)
	{
		a = reg%9;//get new value for sound
		t = pgm_read_word(d+a);
		a = reg%16;//get new value for color
		blue = pgm_read_byte(k+a);
		red = pgm_read_byte(l+a);
		green = pgm_read_byte(m+a);
		j=0;
		
	}

}


void abc(void)
{

	if(e==green)
	{
		PORTB = PORTB&~(1<<0);

	}
	if(e==red)
	{
		PORTB = PORTB&~(1<<1);
	}
	if(e==blue)
	{
		PORTB = PORTB&~(1<<2);
	}
	e++;
}

void delay(int a)
{
	for(int k=0;k<a;k++)
	asm("NOP");
}
